/*
 * Copyright (C) 2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef B3ReduceStrength_h
#define B3ReduceStrength_h

#if ENABLE(B3_JIT)

namespace JSC { namespace B3 {

class Procedure;

// Does strength reduction, constant folding, canonicalization, CFG simplification, DCE, and very
// simple CSE. This phase runs those optimizations to fixpoint. The goal of the phase is to
// dramatically reduce the complexity of the code. In the future, it's preferable to add optimizations
// to this phase rather than creating new optimizations because then the optimizations can participate
// in the fixpoint. However, because of the many interlocking optimizations, it can be difficult to
// add sophisticated optimizations to it. For that reason we have full CSE in a different phase, for
// example.

bool reduceStrength(Procedure&);

} } // namespace JSC::B3

#endif // ENABLE(B3_JIT)

#endif // B3ReduceStrength_h

