/*
 * Copyright (C) 2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef B3SwitchCase_h
#define B3SwitchCase_h

#if ENABLE(B3_JIT)

#include "B3FrequentedBlock.h"
#include <wtf/StdLibExtras.h>
#include <wtf/PrintStream.h>

namespace JSC { namespace B3 {

class SwitchCase {
public:
    SwitchCase()
    {
    }

    SwitchCase(int64_t caseValue, const FrequentedBlock& target)
        : m_caseValue(caseValue)
        , m_target(target)
    {
    }

    explicit operator bool() const { return !!m_target; }

    int64_t caseValue() const { return m_caseValue; }
    FrequentedBlock target() const { return m_target; }
    BasicBlock* targetBlock() const { return m_target.block(); }

    void dump(PrintStream& out) const;

private:
    int64_t m_caseValue;
    FrequentedBlock m_target;
};

} } // namespace JSC::B3

#endif // ENABLE(B3_JIT)

#endif // B3SwitchCase_h

